/*
 * Linear Mask
 *
 * A simple script to quickly create a luminance stretched mask when in linear processing.
 *
 * Copyright (C) 2020, AstroPills
 * Written by Marco Rapino
 */
#include <pjsr/ColorSpace.jsh>
#include <pjsr/UndoFlag.jsh>

#feature-id    Render > LinearMask

#feature-info  A simple script to quickly create a luminance stretched mask when in linear processing.

/*
 * Default STF Parameters by Juan Conejero
 */

// Shadows clipping point measured in sigma units from the main histogram peak.
#define SHADOWS_CLIP -1.25

// Target background in the [0,1] range.
#define TARGET_BKG    0.25


/*
 * Create the mask view from the active window
 */
function createMaskView(window)
{
   console.writeln("Creating mask image...");
   var maskName = window.mainView.id + "_mask";
   var maskWindow = ImageWindow.windowById(maskName);
   if (maskWindow.isWindow)
      maskWindow.close();

   if (window.mainView.image.isColor)
      return createMaskViewFromRGBImage(window, maskName);
   else
      return createMaskFromMonoImage(window, maskName);
}

function createMaskViewFromRGBImage(window, maskName)
{
   var view = window.mainView;
   var P = new ChannelExtraction;
   P.colorSpace = ChannelExtraction.prototype.CIELab;
   P.channels = [ // enabled, id
      [true, maskName],
      [false, ""],
      [false, ""]
   ];
   P.sampleFormat = ChannelExtraction.prototype.SameAsSource;
   P.executeOn(view);
   var maskView = ImageWindow.windowById(maskName).mainView;
   return maskView;
}

function createMaskFromMonoImage(window, maskName)
{
   var view = window.mainView;
   var win = new ImageWindow(
            view.image.width,
            view.image.height,
            1,
            32,
            true,
            false,
            maskName);

   var maskView = win.mainView;
   maskView.beginProcess(UndoFlag_NoSwapFile);
   maskView.image.apply(view.image);
   maskView.endProcess();
   win.bringToFront();
   return maskView;
}


/*
 * STF Auto Stretch routine borrowed from Juan Conejero post on the PixInsight forums.
 */
function applyAutoSTF( view, shadowsClipping, targetBackground )
{
   var stf = new ScreenTransferFunction;

   var n = 1;

   /*
    * Try to find how many channels look as channels of an inverted image.
    * We know a channel has been inverted because the main histogram peak is
    * located over the right-hand half of the histogram. Seems simplistic
    * but this is consistent with astronomical images.
    */
   var invertedChannels = 0;
   for ( var c = 0; c < n; ++c )
   {
      view.image.selectedChannel = c;
      if ( view.image.median() > 0.5 )
         ++invertedChannels;
   }
   view.image.resetSelections();

   if ( invertedChannels < n )
   {
      // Noninverted image

      var c0 = 0;
      var m = 0;
      for ( var c = 0; c < n; ++c )
      {
         view.image.selectedChannel = c;
         var median = view.image.median();
         var avgDev = view.image.avgDev();
         c0 += median + shadowsClipping*avgDev;
         m  += median;
      }
      view.image.resetSelections();
      c0 = Math.range( c0/n, 0.0, 1.0 );
      m = Math.mtf( targetBackground, m/n - c0 );

      stf.STF = [ // c0, c1, m, r0, r1
                  [c0, 1, m, 0, 1],
                  [c0, 1, m, 0, 1],
                  [c0, 1, m, 0, 1],
                  [0, 1, 0.5, 0, 1] ];
   }
   else
   {
      // Inverted image

      var c1 = 0;
      var m = 0;
      for ( var c = 0; c < n; ++c )
      {
         view.image.selectedChannel = c;
         var median = view.image.median();
         var avgDev = view.image.avgDev();
         m  += median;
         c1 += median - shadowsClipping*avgDev;
      }
      view.image.resetSelections();
      c1 = Math.range( c1/n, 0.0, 1.0 );
      m = 1 - Math.mtf( targetBackground, c1 - m/n );

      stf.STF = [ // c0, c1, m, r0, r1
                  [0, c1, m, 0, 1],
                  [0, c1, m, 0, 1],
                  [0, c1, m, 0, 1],
                  [0, 1, 0.5, 0, 1] ];
   }

   console.writeln( "<end><cbr/><b>", view.fullId, "</b>:" );
   for ( var c = 0; c < n; ++c )
   {
      console.writeln( "channel #", c );
      console.writeln( format( "c0 = %.6f", stf.STF[c][0] ) );
      console.writeln( format( "m  = %.6f", stf.STF[c][2] ) );
      console.writeln( format( "c1 = %.6f", stf.STF[c][1] ) );
   }

   var ht = new HistogramTransformation;
   ht.H = [ // c0, m, c1, r0, r1
   [0.00000000, 0.50000000, 1.00000000, 0.00000000, 1.00000000],
   [0.00000000, 0.50000000, 1.00000000, 0.00000000, 1.00000000],
   [0.00000000, 0.50000000, 1.00000000, 0.00000000, 1.00000000],
   [stf.STF[c][0], stf.STF[c][2], 1.00000000, 0.00000000, 1.00000000],
   [0.00000000, 0.50000000, 1.00000000, 0.00000000, 1.00000000]
];

   console.writeln("Stretching mask image...");
   ht.executeOn(view);

   console.writeln( "<end><cbr/>Done!" );
}

function main()
{
   console.show();
   console.writeln("AstroPills - Linear Mask");
   var window = ImageWindow.activeWindow;
   var maskView = createMaskView(window);
   applyAutoSTF( maskView, SHADOWS_CLIP, TARGET_BKG );
}

main();
